/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose, 
including commercial applications, and to alter it and redistribute it freely, 
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/
#ifndef CYBERGLOVES_DEMO_H
#define CYBERGLOVES_DEMO_H

#include "LinearMath/btAlignedObjectArray.h"
#include <Vrui/Vrui.h>
#include <Vrui/Application.h>
#include "DemoApplication.h"

class btBroadphaseInterface;
class btCollisionShape;
class btOverlappingPairCache;
class btCollisionDispatcher;
class btConstraintSolver;
struct btCollisionAlgorithmCreateFunc;
class btDefaultCollisionConfiguration;

///BulletDemo is good starting point for learning the code base and porting.

class BulletDemo : public Vrui::Application, public DemoApplication
{
	float m_Time;
	float m_fCyclePeriod; // in milliseconds
	float m_fMuscleStrength;

	btAlignedObjectArray<class TestRig*> m_rigs;

	//keep the collision shapes, for deletion/cleanup
	btAlignedObjectArray<btCollisionShape*>	m_collisionShapes;

	btBroadphaseInterface*	m_broadphase;

	btCollisionDispatcher*	m_dispatcher;

	btConstraintSolver*	m_solver;

	btDefaultCollisionConfiguration* m_collisionConfiguration;
	//////////////////////	

	public:

	BulletDemo(int& argc,char**& argv);
	virtual ~BulletDemo()
	{
		exitPhysics();
	}
	void spawnTestRig(const btVector3& startOffset, bool bFixed);
	void setMotorTargets(btScalar deltaTime);
	
	/* Vrui */
	virtual void frame(void);
	
    /* Methods from Vrui::Application: */
    virtual void display(GLContextData& contextData) const;
	
	void	initPhysics();

	void	exitPhysics();

	virtual void clientMoveAndDisplay();

	virtual void displayCallback();
	/////////////
	
	void swapBuffers();

	void updateModifierKeys();
};

#endif //BASIC_DEMO_H

